/*
 * Filename : lcd.h
 * Hardware : Controller  -> pic16f690
 *            XTAL        -> INT_RC_OSC ( 8 MHz )
 *
 * I/O      : RS          -> PortC.7
 *            Enable      -> PortC.6
 *            Data456     -> PortC.0-2
 *            Data7       -> PortB.4
 * Compiler : SDCC
 * Author   : sci-3d@hotmail.com
 * Date		: 01/08/06
 * ----------------------------------------------
 * Modified by: Gal Zsolt ( galzsolt at freemail.hu )
 * Date		: 2010.november 6.
 */
#define NO_BIT_DEFINES
#include <pic16f690.h>
#include "lcd.h"

typedef union {
	struct {
		unsigned char LSB:1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char SIGNUM:1;
	};
	struct {
		unsigned char B0:1;
		unsigned char B1:1;
		unsigned char B2:1;
		unsigned char B3:1;
		unsigned char B4:1;
		unsigned char B5:1;
		unsigned char B6:1;
		unsigned char B7:1;
	};
	unsigned char NUMBER;
} bitstruct_t ;

bitstruct_t S_PortA;
bitstruct_t S_PortB;
bitstruct_t S_PortC;
bitstruct_t	LCD_PORT;

/***************************************************
 * Prototype(s)                                    *
 ***************************************************/
void LCD_delay(unsigned char ms);
void LCD_enable();
void LCD_command(unsigned char command);
void LCD_putc(unsigned char ascii);
void LCD_puts(unsigned char *lcd_string);
void LCD_init( void );

void PORT_set( void );
void LCD_nybble(unsigned char command);
void Delay_160( void );
void Delay_5000( void );

/***************************************************
 * Sources                                         *
 ***************************************************/

void Delay_160()                         //  Delay 160 usecs at 4MHz
{
  __asm
  pagesel $
  movlw  .256 - ( .160 / .4 )
  addlw  1
  btfss  STATUS,0
   goto  $-2
  __endasm;
}

void Delay_5000()                       //  Delay 5 msecs at 4MHz
{
  unsigned char i;

  for( i=1 ; i< 25 ; i++ )
  {
     __asm
	 pagesel $
	 movlw  .256 - ( .200 / .4 )          // Delay 200usec at 4MHz
     addlw  1
     btfss  STATUS,0
     goto  $-2
     __endasm;
  }

}

void LCD_delay(unsigned char ms)
{
	unsigned char n;
	unsigned int i;
	for (n=0; n<ms; n++)
	{
		for (i=0; i<LCD_DELAY; i++); /* For some ms */
	}
}

void LCD_enable()
{
    LCD_EN = 1;
    LCD_EN = 0;
}

void PORT_set( void )
{
	S_PortA.NUMBER = PORTA;
	S_PortB.NUMBER = PORTB;
	S_PortC.NUMBER = PORTC;

	LCD_D7 = LCD_PORT.B7;
	LCD_D6 = LCD_PORT.B6;
	LCD_D5 = LCD_PORT.B5;
	LCD_D4 = LCD_PORT.B4;

	PORTA = S_PortA.NUMBER;
	PORTB = S_PortB.NUMBER;
	PORTC = S_PortC.NUMBER;
}

void LCD_nybble(unsigned char command)
{
	LCD_PORT.NUMBER = (LCD_PORT.NUMBER & 0x0F)|((command<<4) & 0xF0);
	PORT_set();
    LCD_enable();
}

void LCD_command(unsigned char command)
{
    LCD_RS = 0;
	LCD_PORT.NUMBER = (LCD_PORT.NUMBER & 0x0F)|(command & 0xF0);
	PORT_set();
    LCD_enable();
	LCD_PORT.NUMBER = (LCD_PORT.NUMBER & 0x0F)|((command<<4) & 0xF0);
	PORT_set();
    LCD_enable();
    Delay_160();
}

void LCD_putc(unsigned char ascii)
{
    LCD_RS = 1;
	LCD_PORT.NUMBER = (LCD_PORT.NUMBER & 0x0F)|(ascii & 0xF0);
	PORT_set();
    LCD_enable();
	LCD_PORT.NUMBER = (LCD_PORT.NUMBER & 0x0F)|((ascii<<4) & 0xF0);
	PORT_set();
    LCD_enable();
    LCD_RS = 0;
    Delay_160();
}

void LCD_puts(unsigned char *lcd_string)
{
	while (*lcd_string)
	{
		LCD_putc(*lcd_string++);
	}
}

void LCD_init()
{

    LCD_EN = 0;
    LCD_RS = 0;

    LCD_nybble(0x00);
    Delay_160();
    LCD_nybble(0x03);
    Delay_5000();
    LCD_nybble(0x03);
    Delay_160();
    LCD_nybble(0x03);
    Delay_160();
    LCD_nybble(0x02);
    Delay_5000();

    LCD_command(0x28);
    Delay_160();
    LCD_command(0x0C);
    LCD_command(0x06);
    LCD_command(0x01); /* Clear */
    LCD_delay(255);
}
