#define NO_BIT_DEFINES
#include <pic16f690.h>
#include "lcd.h"

typedef union {
	struct {
		unsigned char LSB:1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char :1;
		unsigned char SIGNUM:1;
	};
	struct {
		unsigned char B0:1;
		unsigned char B1:1;
		unsigned char B2:1;
		unsigned char B3:1;
		unsigned char B4:1;
		unsigned char B5:1;
		unsigned char B6:1;
		unsigned char B7:1;
	};
	unsigned char NUMBER;
} bitstruct_t ;

typedef unsigned int word;
word __at 0x2007 CONFIG = _INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_ON & _CPD_OFF & _BOR_ON & _IESO_ON & _FCMEN_ON;

extern void LCD_command(unsigned char command);
extern void LCD_putc(unsigned char ascii);
extern void LCD_puts(unsigned char *lcd_string);
extern void LCD_init( void );
extern void Delay_5000( void );

void io_init( void );
void serial_init( void );
void speed_init( void );

volatile unsigned char ReceivedChar,SecondChar;
volatile bitstruct_t ReceiverStatus,DisplayStatus;

void isr() __interrupt 0
{
	if( !ReceiverStatus.B0 )
	{
		ReceivedChar = RCREG;
		ReceiverStatus.B0 = 1;
	}
	else
	{
		SecondChar = RCREG;
		ReceiverStatus.B1 = 1;
	}
}

void io_init( void )
{
	ANSEL  = 0;	// Analog egységek kikapcsolva
	ANSELH = 0;
	TRISA_bits.TRISA2 = 0;	// Háttérvilágítás

	TRISB_bits.TRISB4 = 0;	// LCD	DB7
	TRISB_bits.TRISB5 = 1; // Fogadott soros adat
	TRISB_bits.TRISB6 = 1;
	TRISB_bits.TRISB7 = 1;

	TRISC_bits.TRISC0 = 0;	// LCD 	DB4
	TRISC_bits.TRISC1 = 0;	// LCD	DB5
	TRISC_bits.TRISC2 = 0;	// LCD	DB6
	TRISC_bits.TRISC3 = 1;
	TRISC_bits.TRISC4 = 1;
	TRISC_bits.TRISC5 = 0;	// LCD contrast
	TRISC_bits.TRISC6 = 0;	// LCD	E
	TRISC_bits.TRISC7 = 0; 	// LCD	RS

	PR2 = 255;				// A kontraszt alaphelyzetben a maximumon
	T2CON_bits.TMR2ON = 1;
	CCP1CON = 63;
	CCPR1L = 255;
}

void serial_init( void )
{
	ReceiverStatus.NUMBER = 0;

	SPBRGH = 0;
	SPBRG  = 25;			// 8MHz - 19200 baud

	TXSTA_bits.SYNC = 0;
	TXSTA_bits.BRGH = 1;
	BAUDCTL_bits.BRG16 = 0;
	RCSTA_bits.SPEN = 1;
	RCSTA_bits.CREN = 1;	// Soros vevő egység bekapcsolása

	PIE1_bits.RCIE = 1;
	INTCON_bits.PEIE = 1;
	INTCON_bits.GIE = 1;	// Megaszakítások engedélyezése
}

void speed_init( void )
{
	OSCCON_bits.IRCF0 = 1;	// 8MHz-es sebesség beállítása
	OSCCON_bits.IRCF1 = 1;
	OSCCON_bits.IRCF2 = 1;
}

void main()
{
	unsigned char i = 0;
	unsigned char ch;
	unsigned char InPosition = 0;
	unsigned char OutPosition = 0;
	unsigned char Buffer[32];
	bitstruct_t DisplayStatus;

	DisplayStatus.NUMBER = 0;
	for( i = 0 ; i< 32 ; i++ ) Buffer[i] = 32;

	io_init();
	LCD_init();
	LCD_puts("Soros LCD V0.1");
	speed_init();
	serial_init();

	while( !ReceiverStatus.B0 );
	LCD_clear();
	while( 1 )
	{
		LCD_putc( Buffer[ OutPosition ] );
		OutPosition++;
		if( OutPosition == 16 ) LCD_row2();
		if( OutPosition == 32 )
		{
			LCD_row1();
			OutPosition = 0;
		}

		if( ReceiverStatus.B0 )
		{

			ch = ReceivedChar;
			if( !ReceiverStatus.B1 ) ReceiverStatus.B0 = 0;
			else
			{
				ReceiverStatus.B1 = 0;
				ReceivedChar = SecondChar;
			}


			if( DisplayStatus.B3 )
			{
				DisplayStatus.B3 = 0;
				if( ch == 0 )
				{
					InPosition = 0;
				}
			}
			else
			if( ( 31 < ch ) && ( ch < 127 ) || ( ch > 128 ) )
			{
				Buffer[InPosition] = ch;
				InPosition++;
				if( InPosition == 32 )
				{
					InPosition = 16;
				}
			}

			if( ch == 13 )
			{
				if( InPosition < 16 )
				{
					InPosition = 0;
				}
				else
				if( ( InPosition >= 16  ) && ( InPosition <32 ) )
				{
					InPosition = 16;
				}
			}
			if( ch == 27 ) DisplayStatus.B3 = 1;
		}
	}
}

